@@ -11,6 +11,7 @@
 # renfile_exif_date.pl
 # version 0.2 (2010/December/06)
 # version 0.3 (2011/January/04)
+# version 0.4 (2011/January/05)
 #
 # GNU GPL Free Software
 #
@@ -41,8 +42,12 @@
 use strict;
 use warnings;
 use utf8;
+
+my $flag_os = 'linux';	# linux/windows
+my $flag_charcode = 'utf8';		# utf8/shiftjis
+
 use POSIX;
-use Gtk2 qw/-init/;
+use Gtk2 qw/-init/;	# Windowsの場合は、この行をコメントアウト
 use Image::ExifTool;
 use File::Basename;
 use Encode::Guess qw/euc-jp shiftjis iso-2022-jp/;	# 必要ないエンコードは削除すること
@@ -51,25 +56,47 @@ use Encode::Guess qw/euc-jp shiftjis iso-2022-jp/;	# 必要ないエンコード
 #use Data::HexDump;
 
 
-binmode( STDOUT, ":utf8" ); # "Wide character in print at ..." 警告を抑止
-#binmode(STDOUT, "encoding(sjis)");	# Windowsのコマンドラインではsjisになる
+#binmode( STDOUT, ":utf8" ); # "Wide character in print at ..." 警告を抑止
+if($flag_charcode eq 'utf8'){
+	binmode(STDIN, ":utf8");
+	binmode(STDOUT, ":utf8");
+	binmode(STDERR, ":utf8");
+}
+if($flag_charcode eq 'shiftjis'){
+	binmode(STDIN, "encoding(sjis)");	# Windowsのコマンドラインではsjisになる
+	binmode(STDOUT, "encoding(sjis)");	# Windowsのコマンドラインではsjisになる
+	binmode(STDERR, "encoding(sjis)");	# Windowsのコマンドラインではsjisになる
+}
 
 my $strTargetDir = $ENV{'HOME'};		# 対象ディレクトリ
 my $flag_date_type = 'YYYY-mm-DD';	# 日時形式
 my $flag_change_lowercase = 1;		# ファイル名を小文字に変換する
 
 my @arrImageFiles = ();		# 画像ファイルを格納する配列
-my @arrFileScanMask = ('*.jpg', '*.jpeg', '*.JPG', '*.JPEG');	# 処理対象
-			# Windowsの場合は、大文字の .JPG .JPEG の記述不要
+
+my @arrFileScanMask = undef;
+if($flag_os eq 'linux'){
+	@arrFileScanMask = ('*.jpg', '*.jpeg', '*.JPG', '*.JPEG');	# 処理対象
+}
+if($flag_os eq 'windows'){
+	@arrFileScanMask = ('*.jpg', '*.jpeg');	# 処理対象
+				# Windowsの場合は、大文字の .JPG .JPEG の記述不要
+}
+my @arrKnownExt = ('.jpg', '.jpeg', '.JPG', '.JPEG');
 
 my $flag_gui = 0;
+my $flag_fname_style = 'date-base';		# date-base, base-date, date : ファイル名決定で日時と元ファイル名の利用方法
+my $flag_undo_mode = 0;		# 0:OFF, 1:ON
 my $flag_verbose = 1;
 
 my $return_code = undef;
 
 # GUIスイッチを引数に起動されたとき（Gtk2でのユーザインターフェース対応）
 if(defined($ARGV[0]) && !(lc($ARGV[0]) cmp '-gui')){ $flag_gui = 1; }
-else{ print("\n".basename($0)." - ファイル名にExifより日時を付加するスクリプト\nGUI版は -gui スイッチで起動できます\n\n"); }
+else{
+	print("\n".basename($0)." - ファイル名にExifより日時を付加するスクリプト\n".
+		"GUI版は -gui スイッチで起動できます\n\n");
+}
 
 # ディクトリ、日時形式の入力
 if($flag_gui == 1){ $return_code = sub_user_input_init_gui(); }
@@ -78,13 +105,15 @@ if($return_code == 0){ exit(); }
 
 # 対象ファイル一覧を @arrImageFiles 配列に格納する
 sub_scan_imagefiles();
+sub_sort_imagefiles();
 
 # 対象ファイル一覧を表示して、ファイル名変更を実行するかどうかユーザ選択させる
 my $strPreview = sub_rename_main('preview');
 
 if($flag_gui == 1){
 	if(length($strPreview) > 0){
-		if(sub_display_message($strPreview, "処理内容のプレビュー", "ファイル名変更処理をしますか？", "処理開始", "中止") == 0){ exit(); }
+		if(sub_display_message($strPreview, "処理内容のプレビュー", "ファイル名変更処理をしますか？",
+			 "処理開始", "中止") == 0){ exit(); }
 	}
 	else {
 		sub_display_message("", "指定されたフォルダには jpeg ファイルが見つかりませんでした", "", "", "終了");
@@ -97,7 +126,7 @@ else{
 		printf("ファイル名変更処理をしますか？ (Y/N) [N] : ");
 		$_ = <STDIN>;
 		chomp($_);
-		if(length($_)<=0 || uc($_) ne 'Y'){ die("キャンセルしました\n"); }
+		if(length($_)<=0 || uc($_) ne 'Y'){ die("User Cansel(キャンセルしました)\n"); }
 	}
 	else {
 		die("対象となるファイルが見つからないため終了します\n");
@@ -171,9 +200,22 @@ sub sub_user_input_init_gui {
 	$combobox_datetype->append_text('年月日（YYYY-mm-DD-）');
 	$combobox_datetype->append_text('年月日時分（YY-mm-DD-HHMM）');
 	$combobox_datetype->append_text('年月日時分（YYYY-mm-DD-HHMM）');
-	$combobox_datetype->set_active(5);
+	$combobox_datetype->set_active(0);
 	$hbox_datetype->add($combobox_datetype);
 
+	# ファイル名形式（コンボボックス）
+	my $hbox_filenametype = Gtk2::HBox->new();
+	$vbox->add($hbox_filenametype);
+
+	my $label_filenametype = Gtk2::Label->new("ファイル名形式");
+	$hbox_filenametype->add($label_filenametype);
+	
+	my $combobox_filenametype = Gtk2::ComboBox->new_text();
+	$combobox_filenametype->append_text('日時-元ファイル名（YMD-basename.jpg）');
+	$combobox_filenametype->append_text('元ファイル名-日時（basename-YMD.jpg）');
+	$combobox_filenametype->append_text('日時（YMD.jpg）');
+	$combobox_filenametype->set_active(0);
+	$hbox_filenametype->add($combobox_filenametype);
 
 	# ファイル名の小文字変換（チェックボックス）
 	my $hbox_lc = Gtk2::HBox->new();
@@ -183,6 +225,14 @@ sub sub_user_input_init_gui {
 	if($flag_change_lowercase == 1){ $check_lc->set_active(1); }
 	$hbox_lc->add($check_lc);
 
+	# Undoモード（チェックボックス）
+	my $hbox_undo = Gtk2::HBox->new();
+	$vbox->add($hbox_undo);
+
+	my $check_undo = Gtk2::CheckButton->new_with_label("Undoモード（ファイル名を元に戻す）");
+	if($flag_undo_mode == 1){ $check_undo->set_active(1); }
+	$hbox_undo->add($check_undo);
+
 
 	# 実行・閉じるボタン
 	my $hbox_buttons = Gtk2::HBox->new();
@@ -193,7 +243,7 @@ sub sub_user_input_init_gui {
 		# 実行ボタンが押されたときのコールバック関数
 		$strTargetDir = $entry_dir->get_text();
 		if(substr($strTargetDir,-1) ne '/'){ $strTargetDir .= '/'; }	# ディレクトリは / で終わるように修正
-#		$strTargetDir = Encode::encode('utf8', $strTargetDir);	# Perl内部エンコードからUTF-8に変換し、かつ「UTF-8フラグを取る」
+
 		my $nSelected = $combobox_datetype->get_active();
 		if($nSelected == 0){ $flag_date_type = 'YYmmDD'; }
 		elsif($nSelected == 1){ $flag_date_type = 'YYYYmmDD'; }
@@ -203,8 +253,18 @@ sub sub_user_input_init_gui {
 		elsif($nSelected == 5){ $flag_date_type = 'YYYY-mm-DD'; }
 		elsif($nSelected == 6){ $flag_date_type = 'YY-mm-DD-HHMM'; }
 		elsif($nSelected == 7){ $flag_date_type = 'YYYY-mm-DD-HHMM'; }
+
+		$nSelected = $combobox_filenametype->get_active();
+		if($nSelected == 0){ $flag_fname_style = 'date-base'; }
+		elsif($nSelected == 1){ $flag_fname_style = 'base-date'; }
+		elsif($nSelected == 2){ $flag_fname_style = 'date'; }
+
 		if($check_lc->get_active() == 1){ $flag_change_lowercase = 1; }
 		else{ $flag_change_lowercase = 0; }
+
+		if($check_undo->get_active() == 1){ $flag_undo_mode = 1; }
+		else{ $flag_undo_mode = 0; }
+
 		$nReturn = 1;
 		Gtk2->main_quit();	# ウインドウのメインループを終了する
 	});
@@ -358,29 +418,60 @@ sub sub_user_input_init {
 		else{ die("終了（理由：ディレクトリが入力されませんでした）\n"); }
 	}
 	if(substr($_,-1) ne '/'){ $_ .= '/'; }	# ディレクトリは / で終わるように修正
-	unless(-d $_){ die("終了（理由：ディレクトリ ".$_." が存在しません）\n"); }
+	unless(-d sub_conv_to_local_charset($_)){ die("終了（理由：ディレクトリ ".$_." が存在しません）\n"); }
 	unless($_ =~ m/^\// || $_ =~ m/^\.\//){ $strTargetDir = "./".$_; }
 	else{ $strTargetDir = $_; }
 	$strTargetDir = sub_conv_to_flagged_utf8($strTargetDir);
 	print("対象ディレクトリ : " . $strTargetDir . "\n\n");
 
 
-	print("日時形式の選択\n1.YYmmDD\n2.YYYYmmDD\n3.YYmmDDHHMM\n4.YYYYmmDDHHMM\n".
-		"5.YY-mm-DD\n6.YYYY-mm-DD\n7.YY-mm-DD-HHMM\n8.YYYY-mm-DD-HHMM\n(1〜8) [1] : ");
+	printf("元のファイル名に復元するUndoモードで実行しますか？ (Y/N) [N] : ");
+	$_ = <STDIN>;
+	chomp;
+	if(length($_)<=0 || uc($_) eq 'N'){
+		$flag_undo_mode = 0;
+		print("Undoモード : OFF（通常のExifリネームを行います）\n\n");
+	}
+	else {
+		$flag_undo_mode = 1;
+		print("Undoモード : ON（元のファイル名に復元します）\n\n");
+	}
+
+
+	if($flag_undo_mode == 0){
+		print("日時形式の選択\n1.YYmmDD\n2.YYYYmmDD\n3.YYmmDDHHMM\n4.YYYYmmDDHHMM\n".
+			"5.YY-mm-DD\n6.YYYY-mm-DD\n7.YY-mm-DD-HHMM\n8.YYYY-mm-DD-HHMM\n(1-8) [1] : ");
+		$_ = <STDIN>;
+		chomp;
+		if(length($_)<=0){ $_ = 1; }
+		if(int($_)<1 || int($_)>8){ die("1-8の範囲外が入力されました\n"); }
+		if($_==1){$flag_date_type = 'YYmmDD'; }
+		elsif($_==2){$flag_date_type = 'YYYYmmDD'; }
+		elsif($_==3){$flag_date_type = 'YYmmDDHHMM'; }
+		elsif($_==4){$flag_date_type = 'YYYYmmDDHHMM'; }
+		elsif($_==5){$flag_date_type = 'YY-mm-DD'; }
+		elsif($_==6){$flag_date_type = 'YYYY-mm-DD'; }
+		elsif($_==7){$flag_date_type = 'YY-mm-DD-HHMM'; }
+		elsif($_==8){$flag_date_type = 'YYYY-mm-DD-HHMM'; }
+
+		print("日付形式 : ".$flag_date_type."\n\n");
+	}
+
+	print("ファイル名形式の選択\n1.日時-元ファイル名.jpg\n2.元ファイル名-日時.jpg\n3.日時.jpg\n".
+		"\n(1-3) [1] : ");
 	$_ = <STDIN>;
 	chomp;
 	if(length($_)<=0){ $_ = 1; }
-	if(int($_)<1 || int($_)>8){ die("1〜8の範囲外が入力されました\n"); }
-	if($_==1){$flag_date_type = 'YYmmDD'; }
-	elsif($_==2){$flag_date_type = 'YYYYmmDD'; }
-	elsif($_==3){$flag_date_type = 'YYmmDDHHMM'; }
-	elsif($_==4){$flag_date_type = 'YYYYmmDDHHMM'; }
-	elsif($_==5){$flag_date_type = 'YY-mm-DD'; }
-	elsif($_==6){$flag_date_type = 'YYYY-mm-DD'; }
-	elsif($_==7){$flag_date_type = 'YY-mm-DD-HHMM'; }
-	elsif($_==8){$flag_date_type = 'YYYY-mm-DD-HHMM'; }
+	if(int($_)<1 || int($_)>3){ die("1-3の範囲外が入力されました\n"); }
+	if($_==1){$flag_fname_style = 'date-base'; }
+	elsif($_==2){$flag_fname_style = 'base-date'; }
+	elsif($_==3){$flag_fname_style = 'date'; }
 
-	print("日付形式 : ".$flag_date_type."\n\n");
+	print("ファイル名形式 : ".$flag_fname_style."\n\n");
+
+	if($flag_fname_style eq 'date' && $flag_undo_mode == 1){
+		die("元のファイル名が失われていますので Undo できません。\n");
+	}
 
 	printf("同時に、ファイル名を小文字に変換しますか？ (Y/N) [Y] : ");
 	$_ = <STDIN>;
@@ -414,27 +505,46 @@ sub sub_scan_imagefiles {
 	}
 
 	# ファイル一覧を得て、配列に格納
-	@arrScan = glob($strScanPattern);
+	@arrScan = glob(sub_conv_to_local_charset($strScanPattern));
 
 	# 各ファイルのディレクトリ名、Exif日時を配列 @arrImageFiles に格納
 	foreach(@arrScan)
 	{
 		if(length($_) <= 0){ next; }
 		$_ = sub_conv_to_flagged_utf8($_);
-		$exifTool->ImageInfo($_);
+		$exifTool->ImageInfo(sub_conv_to_local_charset($_));
 		$tmpDate = $exifTool->GetValue('CreateDate');
 		if(!defined($tmpDate)){ $tmpDate = 0; }	# Exifが無い場合は 0
 		else{
 			my @arrTime_t = split(/,/,$tmpDate);
 			$tmpDate = mktime($arrTime_t[5], $arrTime_t[4], $arrTime_t[3], $arrTime_t[2], $arrTime_t[1]-1, $arrTime_t[0]-1900);
 		}
-		my @arrTemp = ($_, dirname($_), basename($_), $tmpDate);
+		my $str_dirname = dirname($_);
+		unless($str_dirname =~ /\/$/){ $str_dirname .= '/'; }		# ディレクトリ名は末尾が / で終わるよう調整
+		my @arrTemp = ($_,			# [0]: 元ファイルのフルパス
+					$str_dirname,	# [1]: ディレクトリ名
+					basename($_, @arrKnownExt),	# [2]: 拡張子抜きのbasename
+					$tmpDate,		# [3]: 日時unix時刻
+					''				# [4]: 変更後のファイル名
+					);
+
 		push(@arrImageFiles, \@arrTemp);
 	}
 
 }
 
 
+# 対象画像ファイルの配列をソートする
+sub sub_sort_imagefiles {
+
+	# unux時間でソートする
+
+	@arrImageFiles = sort { @$a[3] <=> @$b[3]} @arrImageFiles;
+
+}
+
+
+
 # ファイルのリネーム
 sub sub_rename_main {
 
@@ -446,55 +556,41 @@ sub sub_rename_main {
 
 	$strReturn .= "対象ディレクトリ : ".$arrImageFiles[0][1]."\n\n";
 
-	foreach(@arrImageFiles){
-		if(length($_->[3])<8){ $strReturn .= "--   : ".$_->[2]." (no exif)\n"; }
+	for(my $i=0; $i<=$#arrImageFiles; $i++){
+		if(length($arrImageFiles[$i][3])<8){
+				$strReturn .= "--   : ".$arrImageFiles[$i][2]." (no exif)\n";
+				$arrImageFiles[$i][4] = $arrImageFiles[$i][2].'.jpg';	# 新ファイル名リストに入れる（ファイル名重複検査用）
+			}
 		else{
-			my @tm = localtime($_->[3]);
-			my $strFormat;
-			my $nFormatYear;
-			if($flag_date_type eq 'YYmmDD'){ $strFormat = "%02d%02d%02d"; $nFormatYear = ($tm[5]+1900)%100; }
-			elsif($flag_date_type eq 'YYYYmmDD'){ $strFormat = "%04d%02d%02d"; $nFormatYear = ($tm[5]+1900);}
-			elsif($flag_date_type eq 'YYmmDDHHMM'){ $strFormat = "%02d%02d%02d%02d%02d"; $nFormatYear = ($tm[5]+1900)%100;}
-			elsif($flag_date_type eq 'YYYYmmDDHHMM'){ $strFormat = "%04d%02d%02d%02d%02d"; $nFormatYear = ($tm[5]+1900);}
-			elsif($flag_date_type eq 'YY-mm-DD'){ $strFormat = "%02d-%02d-%02d"; $nFormatYear = ($tm[5]+1900)%100; }
-			elsif($flag_date_type eq 'YYYY-mm-DD'){ $strFormat = "%04d-%02d-%02d"; $nFormatYear = ($tm[5]+1900);}
-			elsif($flag_date_type eq 'YY-mm-DD-HHMM'){ $strFormat = "%02d-%02d-%02d-%02d%02d"; $nFormatYear = ($tm[5]+1900)%100;}
-			elsif($flag_date_type eq 'YYYY-mm-DD-HHMM'){ $strFormat = "%04d-%02d-%02d-%02d%02d"; $nFormatYear = ($tm[5]+1900);}
-			else{ $strFormat = "%02d%02d%02d"; $nFormatYear = ($tm[5]+1900)%100; }
-
-			my $strYMD = sprintf($strFormat, $nFormatYear,		# year
-								$tm[4]+1,		# month
-								$tm[3],		# day
-								$tm[2],		# hour
-								$tm[1]			# sec
-								);
-
-			if($flag_change_lowercase == 1){ $_->[2] = lc($_->[2]); }	# ファイル名の小文字化
+			my $strNewName = sub_make_new_filename($arrImageFiles[$i][2], $arrImageFiles[$i][3]);
+			$arrImageFiles[$i][4] = $strNewName;	# 新ファイル名リストに入れる（ファイル重複検査用）
+			$strNewName = $arrImageFiles[$i][1].$strNewName;
 			
-			my $strNewName = sprintf("%s/%s-%s", $_->[1],		# dir
-								$strYMD,		# YMD
-								$_->[2]		# filename
-								);
-			
-			if(length($strYMD)<length($_->[2]) && substr($_->[2], 0, length($strYMD)) eq $strYMD) {
-				# 改名済みの時は、スキップ
-				$strReturn .= "-- : ".$_->[2]."\n";
+			if($arrImageFiles[$i][0] eq $strNewName){
+				# 変更前後のファイル名が同一の時、何もしない
+				$strReturn .= "---- : ".basename($arrImageFiles[$i][0])." == ".basename($strNewName)."\n";
 			}
 			else{
 				# 改名する
 				if($flag_preview eq 'preview'){
 					# プレビューモード
-					$strReturn .= "変更 : ".$_->[2]." -> ".basename($strNewName)."\n";
+					$strReturn .= "変更 : ".basename($arrImageFiles[$i][0])." -> ".basename($strNewName)."\n";
 					$nCount++;
 				}
 				else {
 					# 改名
-print("debug:".$_->[0]."->".$strNewName."\n");
-					if(rename($_->[0], $strNewName) == 1){
-						$strReturn .= "変更 : ".$_->[2]." -> ".basename($strNewName)."\n";
+					my $strT = '.';
+					for(1 .. 5){ $strT .= sub_rand_char(); }
+					my $strTempFile = $arrImageFiles[$i][0] . $strT;
+
+					if(rename(sub_conv_to_local_charset($arrImageFiles[$i][0]),
+						 sub_conv_to_local_charset($strTempFile)) == 1 && 
+						rename(sub_conv_to_local_charset($strTempFile),
+						 sub_conv_to_local_charset($strNewName)) == 1){
+						$strReturn .= "変更 : ".basename($arrImageFiles[$i][0])." -> ".basename($strNewName)."\n";
 						$nCount++;
 					}
-					else{ $strReturn .= "失敗 : ".$_->[2]."\n"; }
+					else{ $strReturn .= "失敗 : ".basename($arrImageFiles[$i][0])."\n"; }
 				}
 			}
 		}
@@ -502,6 +598,9 @@ print("debug:".$_->[0]."->".$strNewName."\n");
 
 	if($flag_preview eq 'preview'){
 		$strReturn .= "\n".sprintf("%d 個のファイル中、対象画像は %d 個です", $#arrImageFiles + 1, $nCount)."\n";
+		
+		# 新ファイル名（チェック用）をクリア
+		for(my $i=0; $i<=$#arrImageFiles; $i++){ $arrImageFiles[$i][4] = ''; }
 	}
 	else{
 		$strReturn .= "\n".sprintf("%d 個のファイルを改名しました", $nCount)."\n";
@@ -511,6 +610,79 @@ print("debug:".$_->[0]."->".$strNewName."\n");
 }
 
 
+# 改名予定のファイル名と一致しない、新ファイル名を生成する。（末尾に-000から-999の枝番を振る）
+sub sub_make_new_filename{
+	my $strOrgBasename = shift;
+	my $tm = shift;
+	
+	my @arr_basename = split(/-/, $strOrgBasename);
+	if($flag_fname_style eq 'date-base'){ $strOrgBasename = $arr_basename[$#arr_basename]; }
+	if($flag_fname_style eq 'base-date'){ $strOrgBasename = $arr_basename[0]; }
+
+	my @tm = localtime($tm);
+	my $strFormat;
+	my $nFormatYear;
+	if($flag_date_type eq 'YYmmDD'){ $strFormat = "%02d%02d%02d"; $nFormatYear = ($tm[5]+1900)%100; }
+	elsif($flag_date_type eq 'YYYYmmDD'){ $strFormat = "%04d%02d%02d"; $nFormatYear = ($tm[5]+1900);}
+	elsif($flag_date_type eq 'YYmmDDHHMM'){ $strFormat = "%02d%02d%02d%02d%02d"; $nFormatYear = ($tm[5]+1900)%100;}
+	elsif($flag_date_type eq 'YYYYmmDDHHMM'){ $strFormat = "%04d%02d%02d%02d%02d"; $nFormatYear = ($tm[5]+1900);}
+	elsif($flag_date_type eq 'YY-mm-DD'){ $strFormat = "%02d-%02d-%02d"; $nFormatYear = ($tm[5]+1900)%100; }
+	elsif($flag_date_type eq 'YYYY-mm-DD'){ $strFormat = "%04d-%02d-%02d"; $nFormatYear = ($tm[5]+1900);}
+	elsif($flag_date_type eq 'YY-mm-DD-HHMM'){ $strFormat = "%02d-%02d-%02d-%02d%02d"; $nFormatYear = ($tm[5]+1900)%100;}
+	elsif($flag_date_type eq 'YYYY-mm-DD-HHMM'){ $strFormat = "%04d-%02d-%02d-%02d%02d"; $nFormatYear = ($tm[5]+1900);}
+	else{ $strFormat = "%02d%02d%02d"; $nFormatYear = ($tm[5]+1900)%100; }
+
+	my $strYMD = sprintf($strFormat, $nFormatYear,		# year
+						$tm[4]+1,		# month
+						$tm[3],		# day
+						$tm[2],		# hour
+						$tm[1]			# sec
+						);
+
+	if($flag_change_lowercase == 1){ $strOrgBasename = lc($strOrgBasename); }	# ファイル名の小文字化
+
+	# 新ファイル名を決定する
+	my $strNewName = undef;
+	if($flag_undo_mode == 1){
+		$strNewName = $strOrgBasename.'.jpg';
+	}
+	elsif($flag_fname_style eq 'date-base'){
+		$strNewName = sprintf("%s-%s.jpg", $strYMD, $strOrgBasename);
+	}
+	elsif($flag_fname_style eq 'base-date'){
+		$strNewName = sprintf("%s-%s.jpg", $strOrgBasename, $strYMD);
+	}
+	elsif($flag_fname_style eq 'date'){
+		$strNewName = sprintf("%s.jpg", $strYMD);
+	}
+
+	# 内部サブルーチン（ファイル名リスト内で一致すれば1を返す）
+	my $include = sub { $_=shift; for(my $i=0; $i<=$#arrImageFiles; $i++){ if($_ eq $arrImageFiles[$i][4]){ return(1); } } return(0); };
+
+	# 改名予定の新ファイル名リストに一致していれば、枝番を振る
+	for(my $i=1; $i<=1000; $i++){
+		if(&$include($strNewName) == 1)
+		{
+			if($i >= 1000){ die("exceed counter 999 at ".$strOrgBasename."\n"); }
+			if($flag_undo_mode == 1){
+				$strNewName = sprintf("%s-%03d.jpg", $i, $strOrgBasename);
+			}
+			elsif($flag_fname_style eq 'date-base'){
+				$strNewName = sprintf("%s-%03d-%s.jpg", $strYMD, $i, $strOrgBasename);
+			}
+			elsif($flag_fname_style eq 'base-date'){
+				$strNewName = sprintf("%s-%s-%03d.jpg", $strOrgBasename, $strYMD, $i);
+			}
+			elsif($flag_fname_style eq 'date'){
+				$strNewName = sprintf("%s-%03d.jpg", $strYMD, $i);
+			}
+		}
+		else{ last; }
+	}
+	
+	return($strNewName);
+}
+
 # 任意の文字コードの文字列を、UTF-8フラグ付きのUTF-8に変換する
 sub sub_conv_to_flagged_utf8{
 
@@ -562,5 +734,29 @@ sub sub_conv_to_unflagged_utf8{
 }
 
 
+# UTF8から現在のOSの文字コードに変換する
+sub sub_conv_to_local_charset{
+	my $str = shift;
+
+	# UTF8から、指定された（OSの）文字コードに変換する
+	$str = Encode::encode($flag_charcode, $str);
+	
+	return($str);
+}
+
+
+# ランダムな文字を作成する
+sub sub_rand_char {
+
+	my @arrSeedChars = ();
+	
+	push @arrSeedChars, ('a'..'z');
+	push @arrSeedChars, ('A'..'Z');
+	push @arrSeedChars, ('0'..'9');
+
+	return $arrSeedChars[int(rand($#arrSeedChars+1))];
+}
+
+
 # スクリプト終了 EOF
 
